#include <iostream>
#include <string>

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

#include "linkdb.h"

class findBidirectionalLinks : public rawLinkVisitor
{
	public:
		findBidirectionalLinks( linkDB& output ) : count( 0 ), output( output ) {}
			
		virtual void visit( linkDB* source, const md5sum& src, const md5sum& dest )
		{
			// If the document links to itself, do not record this link
			if ( src == dest )
			{
				//cerr << "Warning: links to self" << endl;
				return;
			}
			
			// If this is a bidirectional link
			if ( source->isLinked( dest, src ) )
			{
				/*
				md5sum smaller = src;
				md5sum greater = dest;
				
				// If it is the other way around, swap them
				if ( greater < smaller )
				{
					md5sum swap = smaller;
					smaller = greater;
					greater = swap;					
				}
				
				// Copy link data (one direction, with the "least" md5sum first)
				output.link( smaller, greater );
				*/
				
				// Copy the link in both directions
				output.link( src, dest );
				output.link( dest, src );
			}
			
			++ count;
			if ( count % 1000 == 0 )
			{
				cerr << ".";
				if ( count % 10000 == 0 ) cerr << count << endl;
			}
		}
		
	private:
		unsigned long int count;
		linkDB& output;
};


using std::string;

int main( int argc, char** argv )
{
	// Create a directory for the links:
	const char bidirLinks[] = "/tmp/bidir";
	struct stat fileInfo;
	
	// Find out if the bidirLinks exists
	if ( stat( bidirLinks, &fileInfo ) )
	{
		// Create the directory if it does not exist
		if ( errno == ENOENT )
		{
			if ( mkdir( bidirLinks, 0700 ) )
			{
				cerr << "Could not create " << bidirLinks << endl;
				return 3;
			}
		}
		else
		{
			cerr << "Could not stat " << bidirLinks << endl;
			return 1;
		}
	}
	else if ( ! S_ISDIR( fileInfo.st_mode ) )
	{
		cerr << bidirLinks << " is not a directory" << endl;
		return 2;
	}
	
	linkDB source( "", true );
	linkDB output( bidirLinks );
	
	findBidirectionalLinks visitor( output );
	source.visitLinks( visitor );
	
	return 0;
}
