#include <iostream>
#include <string>
#include <map>

#include "linkdb.h"

using std::string;
using std::multimap;

class findMostLinks : public rawLinkVisitor
{
	public:
		// Initializing the md5sum this way is gross: They should have some "null" state
		findMostLinks() : previous( defaultHash ), count( 0 ), db( NULL ) {}
			
		virtual void visit( linkDB* source, const md5sum& src, const md5sum& dest )
		{
			// If this is the first pass through, initialize some stuff
			if ( db == NULL )
			{
				previous = src;
				count = 1;
				db = source;
				return;
			}
			
			// Same as the last, increment the count
			if ( previous == src )
			{
				++ count;
			}
			else
			{
				addLinkCount();
				count = 0;
				previous = src;
			}
		}
		
		// The destructor is virtual because G++ told me so. Isn't it nice?
		virtual ~findMostLinks()
		{
			multimap<unsigned long, md5sum>::reverse_iterator i = links.rbegin();
			while ( i != links.rend() )
			{
				cout << i->first << "\t" << db->getURLString( i->second ) << endl;
				++ i;
			}
		}
	
	private:
		void addLinkCount()
		{
			if ( count > 0 )
			{
				links.insert( pair<unsigned long, md5sum>( count, previous ) );
			}
		}
	
		static const unsigned char defaultHash[16] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
	
		md5sum previous;
		unsigned long count;
		linkDB* db;
		
		multimap<unsigned long, md5sum> links;
};

void usage()
{
	cout << "./findmostlinks [options]" << endl
		<< "Options:" << endl
		<< "\t-h -?\tHelp: Show this usage message" << endl
		<< "\t-d <dir>\tDirectory: Use the databases in <dir>" << endl;
}

int main( int argc, char** argv )
{
	string outputPath = "";
	
	// -? : help
	// -h : help
	// -d : Use database in directory ...
	char options[] = "?hd:";
	
	char opt = ' ';
	while ( (opt = getopt( argc, argv, options )) != -1 )
	{
		switch ( opt )
		{
			// Usage
			case 'h':
			case '?':
				usage();
				return 0;
				break;
			
			case 'd':
				outputPath = optarg;
				break;				
		}
	}
	
	linkDB db( outputPath, true );

	findMostLinks visitor;
	db.visitLinks( visitor );
	
	return 0;
}
