#ifndef _MD5SUM_H
#define _MD5SUM_H

#include <sstream>
#include <iomanip>

#include "md5.h"

static const unsigned char md5sum_defaultHash[16] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";

class md5sum
{
	public:
		/* Copy constructor. */
		md5sum( const md5sum& sum )
		{
			clone( sum );
		}
		
		md5sum& operator=( const md5sum& rhs )
		{
			clone( rhs );
			return *this;
		}
		
		void clone( const md5sum& sum )
		{
			for ( unsigned int i = 0; i < sum.size(); ++ i )
				digest[i] = sum.digest[i];
		}
		
		/** Copies the given hash to this object */
		explicit md5sum( const unsigned char hash[16] = md5sum_defaultHash )
		{
			for ( unsigned int i = 0; i < size(); ++ i )
				digest[i] = hash[i];
		}
		
		/** Creates a hash from a string */
		explicit md5sum( const std::string& str )
		{
			struct MD5Context md5;
			MD5Init( &md5 );
			MD5Update( &md5, (unsigned char*) str.c_str(), str.size() );
			MD5Final( digest, &md5 );
		}
		
		/** Creates a hash from a C style string */
		// TODO: How do I deal with this and the hash copy constructor above?
		/*
		explicit md5sum( const unsigned char* str )
		{
			// TODO: This could be optimized by MD5Updating the string
			// as we are processing it. For optimal performance,
			// I think that it should get called each time the largest possible CPU
			// buffer chunk has been processed

			// Determine the length of the string.
			unsigned long int length = 0;
			while ( str[length] != '\0' )
				++ length;

			struct MD5Context md5;
			MD5Init( &md5 );
						
			MD5Update( &md5, (const unsigned char*) str, length );
			MD5Final( digest, &md5 );
		}
		*/
	
		// Convert the hash to a hexadecimal string
		std::string toString() const
		{
			std::ostringstream output;
			for ( unsigned int i = 0; i < size(); ++ i )
			{
				// Output numbers in hexadecimal with a width of two and zero padded
				// this way, the number "4" will result in: "04"
				// this also must be set each time, since it?s reset to zero by each << (inserter)
				// the with must be set each time, since it?s reset to zero by each << (inserter)
				output << std::hex << std::setfill( '0' ) << std::setw( 2 ) << (int) digest[i];
			}
		
			return output.str();
		}
		
		const unsigned char* getDigest() const
		{
			return digest;
		}
		
		const unsigned int size() const
		{
			return sizeof( digest ) / sizeof( *digest );
		}
		
		bool operator==( const md5sum& rhs ) const
		{
			for ( unsigned int i = 0; i < size(); ++ i )
				if ( digest[i] != rhs.digest[i] ) return false;
			
			return true;
		}
		
		bool operator<( const md5sum& rhs ) const
		{
			// Locate the most significant byte that differs in magnitude
			unsigned int i = 0;
			while ( i < size() && digest[i] == rhs.digest[i] )
			{
				++ i;
			}
			
			// Compare the most significant byte
			bool retVal = false;
			if ( i < size() && digest[i] < rhs.digest[i] ) retVal = true;
			
			//cerr << toString() << " < " << rhs.toString() << " = " << retVal << endl;
			return retVal;
		}
	
	private:
		unsigned char digest[16];
};

#endif
