#include <iostream>
#include <string>

#include <unistd.h>

#include "linkdb.h"

using std::string;

class printURLs : public urlVisitor
{
	public:
		printURLs() : count( 0 )
		{}
	
		virtual void visit( const md5sum& hash, const string& url, const string& title )
		{
			cout << url;
			if ( title != "" )
				cout << " (" << title << ")";
			cout << endl;
			++ count;
		}
		
		const unsigned long int & getTotal() const
		{
			return count;
		}
		
	private:
		unsigned long int count;
};

class printLinks : public linkVisitor
{
	public:
		virtual void visit( const string& src, const string& dest )
		{
			cout << src << "    " << dest << endl;			
		}	
};

void usage()
{
	cout << "./test-dbm [options]" << endl
		<< "Options:" << endl
		<< "\t-h -?\tHelp: Show this usage message" << endl
		<< "\t-u\tURLs: Print URLs in the database" << endl
		<< "\t-l\tLinks: (Default mode) Print links in the database" << endl
		<< "\t-d <dir>\tDirectory: Use the databases in <dir>" << endl;
}

int main( int argc, char** argv )
{
	string outputPath = "";
	bool linksMode = true;
	
	// -? : help
	// -h : help
	// -u : print urls
	// -l : print links (optional)
	// -d : Use database in directory ...
	char options[] = "?huld:";
	
	char opt = ' ';
	while ( (opt = getopt( argc, argv, options )) != -1 )
	{
		switch ( opt )
		{
			// Usage
			case 'h':
			case '?':
				usage();
				return 0;
				break;
			
			case 'u':
				linksMode = false;
				break;
			
			case 'd':
				outputPath = optarg;
				break;				
		}
	}
	
	linkDB db( outputPath, true );

	if ( linksMode )
	{
		printLinks linkPrint;
		db.visitLinks( linkPrint );
	}
	else
	{
		printURLs visitor;
		db.visitURLs( visitor );
	
		//cout << visitor.getTotal() << " unique URLs in the database" << endl;
	}
	
	return 0;
}
