#include <iostream>
#include <string>

#include <vector>

#include <unistd.h>

#include "linkdb.h"
#include "sitemap.h"

using std::string;


class printSiteURLs : public siteURLVisitor
{
	public:
		printSiteURLs( linkDB& db ) : db( db ), count( 0 ) {}
	
		virtual void visit( const md5sum& hash )
		{
			cout << db.getURLString( hash ) << endl;
			++ count;
		}
	
		const unsigned long int & getTotal() const
		{
			return count;
		}
	
	private:
		linkDB& db;
		unsigned long int count;
};

class printSites : public siteMapVisitor
{
	public:
		printSites( linkDB& db ) : count( 0 ), db( db ) {}
		virtual void visit( const md5sum& site, const Tree& map )
		{
			cout << "site = " << site.toString() << endl;
			
			siteMap::URLCollector visitor;
			map.accept( visitor );
			
			for ( set<md5sum>::const_iterator i = visitor.begin(); i != visitor.end(); ++ i )
			{
				string url = db.getURLString( *i );
				assert( url != "" );
				
				cout << "\t" << url << endl;
			}
			
			
		}
		
		// The destructor is virtual because G++ told me so. Isn't it nice?
		virtual ~printSites()
		{
			cerr << endl << count << " sites processed" << endl;
		}
		
	private:
		// TODO: This recursive function will hog a *lot* of RAM. Oh well.
		void addLinkedURLs( TreeNode* node, set<md5sum>& remainingURLs )
		{
			vector<md5sum> result = db.getLinks( node->getValue() );
			
			for ( vector<md5sum>::const_iterator i = result.begin(); i != result.end(); ++ i )
			{
				// If the URL has not been parented by another site, we'll take it!
				if ( remainingURLs.count( *i ) > 0 )
				{
					remainingURLs.erase( *i );
					TreeNode* child = node->appendChild( *i );
					addLinkedURLs( child, remainingURLs );
				}
			}
		}
		
		unsigned long count;
		linkDB& db;
};

void usage()
{
	cout << "./test-sitemap [options]" << endl
		<< "Options:" << endl
		<< "\t-h -?\tHelp: Show this usage message" << endl
		<< "\t-u\tURLs: (Default mode) Iterate over URLs in the sitemap database" << endl
		<< "\t-s\tSites: Iterate over sites in the sitemap database" << endl
		<< "\t-d <dir>\tDirectory: Use the databases in <dir>" << endl;
}

int main( int argc, char** argv )
{
	string outputPath = "/tmp/bidir";
	bool URLMode = true;
	
	// -? : help
	// -h : help
	// -u : print urls
	// -l : print links (optional)
	// -d : Use database in directory ...
	char options[] = "?husd:";
	
	char opt = ' ';
	while ( (opt = getopt( argc, argv, options )) != -1 )
	{
		switch ( opt )
		{
			// Usage
			case 'h':
			case '?':
				usage();
				return 0;
				break;
			
			case 's':
				URLMode = false;
				break;
			
			case 'd':
				outputPath = optarg;
				break;				
		}
	}
	
	linkDB db( outputPath, true );
	siteMap sitemap( outputPath, true );

	if ( URLMode )
	{
		printSiteURLs visitor( db );
		sitemap.accept( visitor );
		
		//cout << visitor.getTotal() << " URLs in the sitemap database" << endl;
	}
	else
	{
		printSites visitor( db );
		sitemap.accept( visitor );
	}
	
	return 0;
}
